"""This module contains metadata about the project."""
title = 'fahrplan'
version = '0.1b4'
description = 'A SBB/CFF/FFS commandline based timetable client.'
author = 'Danilo Bargen'
author_email = 'gezuru@gmail.com'
license = 'GPLv3'
